package com.jollychic.spark

import com.jollychic.tool.{Calculate, SparkSQlTools, UserGroupRddTools}
import com.jollychic.utils.{ConstantUtils, DateUtils}
import org.apache.commons.lang3.StringUtils
import org.apache.spark.sql.SparkSession

/**
 * 未用未用未用
 */
/**
  * 根据业务过滤指定的用户群体
  */

object TagUserGroupTask {

  def main(args: Array[String]): Unit = {

    if (args.length != 2) {
      System.err.println(
        """
          |Usage: UserGroupFilterTask  must input four parameter
          | 1st  is  the id  of the user group
          | 2st  is  a tabName for save data of user group
        """.stripMargin)
      System.exit(1)
    }
    try {
      val tabs = (args(0), args(1))
      val spark = SparkSession
        .builder()
        .appName("userprofile-TagUserGroupTask")
        .config("spark.serializer", "org.apache.spark.serializer.KryoSerializer")
        .enableHiveSupport()
        .getOrCreate()

      // 读取规则表，并且注册为视图
      SparkSQlTools.readMySqlTab(spark, ConstantUtils.DBS_TABLE, "v_user_group_category")

      import spark.sql
      // 读取规则数据
      val userCategoryRdd = sql(
        s"""
           | SELECT category_id, category_name, category_rule, tag_based,exclude_category, target_system, member_quantity
           | FROM v_user_group_category WHERE category_status = 0 and  category_id = ${tabs._1}
         """.stripMargin).rdd

      // 计算人群参数
      val userGroupParamsRdd = UserGroupRddTools.
        extractUserGroupParams(userCategoryRdd)

      // 获取计算人群参数
      userGroupParamsRdd.collect()
        .foreach(tp => {
          val column = tp._5 match {
            case 0 => "cookieid"
            case 1 => "userid"
          }
          // 时间
          val nowDay = Calculate.theDayForQueryUserGroup()
          // 人群规则
          val userGroupRuler = Calculate.calculateParams4Json(tp._3)
          var sqlStr = ""
          if (StringUtils.isNoneEmpty(tp._7)) {
            // 字段 业务编号, 规则ID, 表
            val exSqltp = Calculate.calculateExcludeCategory(tp._6, tp._7, tabs._2, nowDay)

            sqlStr =
              s"""
                 |SELECT '${tp._1}' AS tagid ,tbs.id ,tbs.target  FROM (
                 |   SELECT t.id ${exSqltp._2},target
                 |   FROM (
                 |     SELECT ${column} as id,target
                 |       FROM ${tp._4} lateral view explode(split('${tp._6}',','))  tagtbs as target
                 |       WHERE data_date = '$nowDay' AND ($userGroupRuler)) t
                 |     ${exSqltp._1}
                 |)) tbs   WHERE (${exSqltp._3}) = 0
                  """.stripMargin
          } else {
            sqlStr =
              s"""
                 |  SELECT '${tp._1}' as tagid ,$column as id, target,'',''
                 |  FROM ${tp._4}  lateral view explode(split('${tp._6}',','))  tagtbs as target
                 |  WHERE data_date = '$nowDay' AND ($userGroupRuler)
                  """.stripMargin
          }

          println(sqlStr)
          // 执行
          import spark.sql
//           注册临时的人群标签集合表
          sql(sqlStr).createTempView("v_all_target_usergroup")
          // 添加默认分区
          (if (StringUtils.isEmpty(tp._6)) "default" else tp._6 ).split(",")
           .foreach(target => {
            sql(
              s"""
                 |INSERT OVERWRITE TABLE dw.dw_profile_tag_usergroup PARTITION (data_date = '${DateUtils.getNDay(-1)}',target_sys = '${target}' ,tagtype = '${tp._1}')
                 |select tagid, id, target,'','' from v_all_target_usergroup where target='${target}'
            """.stripMargin)
          })

          // 处理默认分区问题
          val sqlTarget = if (tp._6.equals("default")) "AND target = 'default'" else  " AND target <> 'default' "

          val checkUserGroupData =
            s"""
               | SELECT count(*) AS cts  FROM dw.dw_profile_tag_usergroup
               | WHERE data_date = '${DateUtils.getNDay(-1)}' AND tagid = ${tp._1}  ${sqlTarget}
            """.stripMargin
          val dataCount = sql(checkUserGroupData).collect()(0).getAs[Long]("cts")
          val dataCountPer = dataCount / tp._6.split(",").length
          SparkSQlTools.noticeUserGroupDataUpdateTime(tp._1, dataCountPer, tp._8)
          SparkSQlTools.noticeUserGroupDataLogs(tp._1, dataCountPer, tp._8, "dw_profile_tag_usergroup", tp._6)
        })
      spark.stop()
    } catch {
      case ex:Exception => {
        println(s" UserGroupTag ： There may be a problem ： ${ex.printStackTrace}")
        throw new RuntimeException ("UserGroupTag ： There may be a problem")
      }
    }
    System.exit(0)
  }
}
