package com.jollychic.tool

import java.util

import com.alibaba.fastjson.{JSON, JSONArray, JSONObject}
import com.jollychic.utils.DateUtils
import org.apache.commons.lang.StringUtils
import org.slf4j.{Logger, LoggerFactory}
import scalikejdbc.config.DBsWithEnv

import scala.collection.JavaConversions._

/**
  * 计算公式
 */
object Calculate {

  private val LOGGER: Logger = LoggerFactory.getLogger(Calculate.getClass)

  private [this] val PARAMS_BANK = " "

  private [this] val  PARMS_IS_NOT_NULL = " IS NOT NULL"

  private [this] val AND =  " AND"

  private [this] val OR =  " OR"

  private [this] val NULL = "null"

  /**
    * 计算过滤条件
    *
    * @param json
    * @return Boolean  true 表示满足条件  false 表示不满足条件
    * @author zhangyong
    */
  def calculateParams4Json(json: String) = {

    var lst = List[String]()
      try {
        val arr: JSONArray = JSON.parseArray(json)
        // 遍历条件数组
        for (t <- arr) {
          val obj = t.asInstanceOf[JSONObject]
          val set = obj.keySet()
          val size = set.size()
          lst = size match {
            case 1 => calculateFilteAnd(set, obj, lst) // AND
            case x if size > 1 => calculateFilteOr(set, obj, lst) // OR
          }
        }
      } catch {
        case ex:Exception =>  if (LOGGER.isErrorEnabled) LOGGER.error(s"解析人群过滤规则Json失败，待解析的规则数据为： ${json}" ,ex)
      }
    lst.mkString(PARAMS_BANK)
  }

  /**
    * 过滤的规则
    * @return String
    * @author zhangyong
    */
 private def calculateFilteAnd(keySet: util.Set[String], obj: JSONObject,lst: List[String]) = {
    var lstNew = lst: List[String]
    val sb = new StringBuffer()
    for (k <- keySet) {
      val v = obj.getString(k)
      // kv 则为  条件
      if (StringUtils.isEmpty(v)) { // 分类标签
        sb.append("tagsmap[\""+k +"\"]"+  PARMS_IS_NOT_NULL + AND)
      } else { // 统计标签
        // " [null,3] "
        val split = v.substring(1, v.length - 1).split(",")
        val str: String = split.contains(NULL) match {
          // null 在左边  null ,123
          case true => if (split(0).equals(NULL)) s" round (CAST (tagsmap['${k}'] AS double),2) <=  ${split(1)} AND" else s" round ( CAST ( tagsmap['${k}']  AS double),2) >=  ${split(0)}  AND"
          case false => s" round ( CAST ( tagsmap['${k}']  AS double),2) >= ${split(0)}  AND round (  CAST ( tagsmap['${k}']  AS double),2)  <= ${split(1)} AND"
        }
        sb.append(str)
      }
    }
    //结果条件
     val resStr:String = if (lst.size == 0 )  subParamsStr(sb.toString,AND)  else  " AND " + subParamsStr(sb.toString,AND)
    lstNew = lst.:+( resStr)
    lstNew
  }

  /**
    * 过滤的规则
    * @return String
    * @author zhangyong
    */
  private def calculateFilteOr(keySet: util.Set[String], obj: JSONObject,lst: List[String]) = {
    var lstNew = lst: List[String]
    val sb = new StringBuffer()
    for (k <- keySet) {
      val v = obj.getString(k)
      // kv 则为  条件
      if (StringUtils.isEmpty(v)) { // 分类标签
        sb.append(" tagsmap['"+k +"']"+  PARMS_IS_NOT_NULL + OR)
      } else { // 统计标签
        // " [null,3] "
        val split = v.substring(1, v.length - 1).split(",")
        val str: String = split.contains("null") match {
          case true => {
            // null 在左边  null ,123
            if (split(0).equals("null")) s" round ( CAST ( tagsmap['${k}'] AS double),2)  <=  ${split(0)} OR " else s" round ( CAST ( tagsmap['${k}'] AS double),2) >=  ${split(0)}  OR"
          }
          case false => s" round ( CAST ( tagsmap['${k}']  AS double),2) >= ${split(0)}  OR round (  CAST ( tagsmap['${k}']  <= ${split(1)} AS double),2)  OR"
        }
        sb.append(str)
      }
    }
    //结果条件
    var resStr:String = if (lst.size == 0 )  " (" + subParamsStr(sb.toString,"OR")  +" ) "  else  " AND ( " + subParamsStr(sb.toString,"OR") +" ) "
   // 合并 OR 条件
    lstNew = lst.:+( resStr)
    lstNew
  }

  /**
    * @param str
    * @param flag
    * Description: 截取合法的条件语句 [截取AND 与 OR]
    * @author zhangyong
   */
  def subParamsStr (str:String,flag:String) = str.substring(0,str.lastIndexOf(flag))


  /**
    * 计算减法人群规则
    * @param target 业务方编号
    * @param tagIds 人群规则编号【多个人群规则 , 隔开】
    * @param tabsName 所在人群的数据表
    * @param dateTime 时间分区
    *
    * @return sql
    * @author zhangyong
    */
  def calculateExcludeCategory (target:String,tagIds:String,tabsName:String,dateTime:String) = {

    // 提取 减法人群ID
    val sqlArr = tagIds.split(",") map {
      tid =>
        val sql =
          s"""
            SELECT id
            FROM $tabsName
            where  data_date = '$dateTime' and  tagid = '$tid'
          """.stripMargin
        sql
    }
    // sql语句
    val sbSql = new StringBuilder()
    val sbCol = new StringBuilder()
    val sbWhere = new StringBuilder()
      for (ids <- 0 until sqlArr.length) {
        //sql
        sbSql.append(s" left join ( ${sqlArr(ids)}) t${ids}  on (t.id =  t${ids}.id ")
        //列 if(t0.userid IS NULL, 0,1)
        sbCol.append(s", IF( t$ids.id IS NULL, 0,1) as t${ids}flag")
        //      sbWhere.append(s" IF ( t${ids}.${column} IS NULL, 0,1)) + ")
        sbWhere.append(s" tbs.t${ids}flag + ")
      }

      // 条件语句
      val strWhere = sbWhere.toString
      val whereSQL = strWhere.substring(0,strWhere.lastIndexOf("+"))
      (sbSql.toString,sbCol.toString, whereSQL)
  }

  /**
    * 计算人群的日期时间
    *   如果今日数据有问题，测试查询 昨天、前天、依次
    * @return ${Int}
    * @author zhangyong
   */
  def theDayForQueryUserGroup(): String = {

    DBsWithEnv("monitor").setupAll()
    import scalikejdbc._
    val dateFun = (rs: WrappedResultSet) => rs.string("process_date").replace("-","")
    val dateOpt =  DB readOnly {
      implicit  session=>
        sql" SELECT process_date  FROM  tag_monitor_state WHERE  stage = 'Cluster' AND  state = 0 ORDER  BY process_date DESC limit 1"
          .map(dateFun).single().apply()
    }
    dateOpt.getOrElse(DateUtils.getNDayYMD())
  }

}


