package com.jollychic.tool

import java.sql.DriverManager

import com.jollychic.utils.{ConstantUtils, DateUtils}
import org.apache.spark.sql.SparkSession

object SparkSQlTools {

  /**
    * 使用SparkSQl读取表注册未视图
    * @param tbsName 读取的表名
    * @param vName 视图名称
    */
  def readMySqlTab(spark:SparkSession,tbsName:String,vName:String) ={
    val ruleDF = spark
      .read
      .format("jdbc")
      .option("driver",ConstantUtils.DBS_MONITOR__DRIVER)
      .option("url", ConstantUtils.DBS_URL)
      .option("dbtable", tbsName)
      .option("user", ConstantUtils.DBS_USER)
      .option("password", ConstantUtils.DBS_PASSWORD)
      .load()
    // 注册视图
    ruleDF.createOrReplaceTempView(vName)
    ruleDF
  }



  /**
   * Oracle
   * @param tbsName 读取的表名
   * @param vName 视图名称
   */
  def readOracleTab(spark:SparkSession,tbsName:String,vName:String) ={
    val ruleDF = spark
      .read
      .format("jdbc")
      .option("driver",ConstantUtils.ORACALDB_DRIVER)
      .option("url", ConstantUtils.ORACALDB_URL)
      .option("dbtable", tbsName)
      .option("user", ConstantUtils.ORACALDB_USER)
      .option("password", ConstantUtils.ORACALDB_PASSWORD)
      .load()
    // 注册视图
    ruleDF.createOrReplaceTempView(vName)
    ruleDF
  }

  /**
    * 通知分群分组的更新时时间
    * （更新mysql数据库）
    * @param tagId 人群规则ID
    *
    */
  def noticeUserGroupDataUpdateTime(tagId:Int,dataCount:Long, memberQuantity:Long): Unit ={
    // 预计人群数量
//    val rate = math.abs((dataCount - memberQuantity*ConstantUtils.PREDIC_FACTOR)/(memberQuantity*ConstantUtils.PREDIC_FACTOR))

    val execRes = if (dataCount == memberQuantity ) 1 else 2
    // 更新
    val dateTime = DateUtils.nowDateTime
    val conn =  DriverManager.getConnection(ConstantUtils.DBS_URL,ConstantUtils.DBS_USER,ConstantUtils.DBS_PASSWORD)
    // 执行结果
    val sqlupdate =
      s"""
         | UPDATE ${ConstantUtils.DBS_TABLE}
         | SET  category_exec_status = ?, `usergroup_update_time` = ?
         | WHERE (`category_id` = ?)
                """.stripMargin
    val ps = conn.prepareStatement(sqlupdate)
    ps.setInt(1,execRes)
    ps.setString(2,dateTime)
    ps.setInt(3,tagId)
    // 执行sql
    ps.executeUpdate()
  }


  /**
    * 记录的人群分组信息更新时时间
    * （更新mysql数据库）
    * @param tagId 人群规则ID
    *
    */
  def noticeUserGroupDataLogs(tagId:Int,dataCount:Long, memberQuantity:Long,desc:String,target:String): Unit ={
    // 预计人群数量

    // 更新
    val dateTime = DateUtils.nowDateTime
    val conn =  DriverManager.getConnection(
          ConstantUtils.DBS_MONITOR_URL,
          ConstantUtils.DBS_MONITOR_USER,
          ConstantUtils.DBS_MONITOR_PASSWORD)
    // 执行结果
//    val sqlupdate =
//      s"""
//         | UPDATE ${ConstantUtils.DBS_TABLE}
//         | SET  category_exec_status = ?, `usergroup_update_time` = ?
//         | WHERE (`category_id` = ?)
//                """.stripMargin

    val sqlInsert =
      s"""
        |INSERT INTO ${ConstantUtils.DBS_MONITOR_TABLE} ( `tagid`, `update_time`, `total_count`, `predic_val`, `type`,`target_system`)
        |VALUES (?, ?, ?, ?,'${desc}','${target}');
      """.stripMargin

    val ps = conn.prepareStatement(sqlInsert)
    ps.setInt(1,tagId)
    ps.setString(2,dateTime)
    ps.setLong(3,dataCount)
    ps.setLong(4,memberQuantity)
    // 执行sql
    ps.executeUpdate()
  }

}
