package com.jollychic.tool

import com.jollychic.utils.ConstantUtils
import org.apache.commons.lang3.StringUtils
import org.apache.spark.rdd.RDD
import org.apache.spark.sql.Row

/**
  * 用户人群分组工具
  */
object UserGroupRddTools {

  /**
    * 提取人群分组的相关参数
    * @param userCategoryRdd 人群规则RDD
    */
  def extractUserGroupParams (userCategoryRdd:RDD[Row]) = {

    // 计算人群参数
    val userGroupParamsRdd = userCategoryRdd.map(row => {
      // 查询适配 tag_based 关联   标签，0：cookieId，1：userId
      val tagBased = row.getAs[Int]("tag_based")
      // 人群标签ID
      val categoryId = row.getAs[Int]("category_id")
      // 人群标签
      val categoryName = row.getAs[String]("category_name")
      //人群区分规则（json）
      val ategoryRule = row.getAs[String]("category_rule")
      // 业务方编号
      var target = row.getAs[String]("target_system")
      if (StringUtils.isEmpty(target)) target = "default"
      //数据存储位置
      val dataTabs = tagBased match {
        case 0 => (ConstantUtils.TASK_FILTED_USER_GROUP_IN_CK)
        case 1 => (ConstantUtils.TASK_FILTED_USER_GROUP_IN_US)
      }
      //预计人群数量
      val memberQuantity = row.getAs[Int]("member_quantity")

      // 减法人群
      val exCid = row.getAs[String]("exclude_category")
      (categoryId,categoryName,ategoryRule,dataTabs,tagBased,target,exCid,memberQuantity)
    })
    userGroupParamsRdd
  }


}
